# -*- coding: utf-8 -*-
import os
from io import open
from setuptools import setup
from setuptools import find_packages

here = os.path.abspath(os.path.dirname(__file__))

with open(os.path.join(here, "README.md"), "r", encoding="utf-8") as fobj:
    long_description = fobj.read()

with open(os.path.join(here, "requirements.txt"), "r", encoding="utf-8") as fobj:
    requires = [x.strip() for x in fobj.readlines() if x.strip()]


setup(
    name="simqsvr",
    version="0.1.1",
    description="SIMQ服务器。基于redis的消息队列，通过web接口提供对外服务。",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Chen Han",
    maintainer="Chen Han",
    license="Apache License, Version 2.0",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "License :: OSI Approved :: Apache Software License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3 :: Only",
    ],
    keywords=["django"],
    install_requires=requires,
    packages=find_packages(".", exclude=["simqtest", "simqtest.migrations"]),
    py_modules=["manage_simqsvr"],
    zip_safe=False,
    include_package_data=True,
    entry_points={
        "console_scripts": [
            "manage-simqsvr = manage_simqsvr:main",
        ]
    },
)
